OPTION_TYPE_CALL = 0
OPTION_TYPE_PUT = 1
OPTION_TYPE_UNDEFINED = -1


INSTRUMENT_TYPE_STOCK = 'stock'
INSTRUMENT_TYPE_FUTURE = 'future'
INSTRUMENT_TYPE_OPTION = 'option'
INSTRUMENT_TYPE_UNDEFINED = 'undefined'


INSTRUMENT_EXECUTION_BUY = 1
INSTRUMENT_EXECUTION_SELL = -1
